@@ -18,3 +18,5 @@ CONFIG_DWH_ACCESS_TOKEN = "dwh_access_token"
 CONFIG_DBT_REPO = "dbt_repo"
 CONFIG_DBT_PROFILE = "dbt_profile"
 CONFIG_DBT_TARGET = "dbt_target"
+CONFIG_DBT_CLOUD_JOB_ID = "dbt_cloud_job_id"
+CONFIG_DBT_CLOUD_SERVICE_TOKEN = "dbt_cloud_service_token"
