@@ -21,10 +21,12 @@ DATAFLOW_NODE_COMBINE_METRICS_ID_PREFIX = "cbm"
 DATAFLOW_NODE_CONSTRAIN_TIME_RANGE_ID_PREFIX = "ctr"
 DATAFLOW_NODE_SET_MEASURE_AGGREGATION_TIME = "sma"
 DATAFLOW_NODE_SEMI_ADDITIVE_JOIN_ID_PREFIX = "saj"
+DATAFLOW_NODE_JOIN_TO_TIME_SPINE_ID_PREFIX = "jts"
 
 SQL_EXPR_COLUMN_REFERENCE_ID_PREFIX = "cr"
 SQL_EXPR_COMPARISON_ID_PREFIX = "cmp"
 SQL_EXPR_FUNCTION_ID_PREFIX = "fnc"
+SQL_EXPR_PERCENTILE_ID_PREFIX = "perc"
 SQL_EXPR_STRING_ID_PREFIX = "str"
 SQL_EXPR_NULL_PREFIX = "null"
 SQL_EXPR_LOGICAL_OPERATOR_PREFIX = "lo"
@@ -34,6 +36,8 @@ SQL_EXPR_CAST_TO_TIMESTAMP_PREFIX = "ctt"
 SQL_EXPR_DATE_TRUNC = "dt"
 SQL_EXPR_RATIO_COMPUTATION = "rc"
 SQL_EXPR_BETWEEN_PREFIX = "betw"
+SQL_EXPR_WINDOW_FUNCTION_ID_PREFIX = "wfnc"
+SQL_EXPR_GENERATE_UUID_PREFIX = "uuid"
 
 SQL_PLAN_SELECT_STATEMENT_ID_PREFIX = "ss"
 SQL_PLAN_TABLE_FROM_CLAUSE_ID_PREFIX = "tfc"
@@ -43,6 +47,7 @@ EXEC_NODE_NOOP = "noop"
 EXEC_NODE_WRITE_TO_TABLE = "wtt"
 
 DATAFLOW_PLAN_PREFIX = "dfp"
+OPTIMIZED_DATAFLOW_PLAN_PREFIX = "dfpo"
 SQL_QUERY_PLAN_PREFIX = "sqp"
 EXEC_PLAN_PREFIX = "ep"
 
