@@ -2,13 +2,15 @@
 
 from __future__ import annotations
 
+import logging
 import textwrap
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import List, TypeVar, Generic, Optional, Sequence, Tuple, Union
+from typing import List, TypeVar, Generic, Optional, Sequence, Tuple, Union, Type
 
 import jinja2
 
+from metricflow.aggregation_properties import AggregationType
 from metricflow.constraints.time_constraint import TimeRangeConstraint
 from metricflow.dag.id_generation import (
     DATAFLOW_NODE_AGGREGATE_MEASURES_ID_PREFIX,
@@ -25,6 +27,7 @@ from metricflow.dag.id_generation import (
     DATAFLOW_NODE_COMBINE_METRICS_ID_PREFIX,
     DATAFLOW_NODE_CONSTRAIN_TIME_RANGE_ID_PREFIX,
     DATAFLOW_NODE_SET_MEASURE_AGGREGATION_TIME,
+    DATAFLOW_NODE_JOIN_TO_TIME_SPINE_ID_PREFIX,
 )
 from metricflow.dag.mf_dag import DagNode, DisplayedProperty, MetricFlowDag, NodeId
 from metricflow.dataflow.builder.partitions import (
@@ -33,24 +36,27 @@ from metricflow.dataflow.builder.partitions import (
 )
 from metricflow.dataflow.sql_table import SqlTable
 from metricflow.dataset.dataset import DataSet
-from metricflow.model.objects.metric import CumulativeMetricWindow
-from metricflow.aggregation_properties import AggregationType
+from metricflow.model.objects.metric import MetricTimeWindow
 from metricflow.object_utils import pformat_big_objects
 from metricflow.references import TimeDimensionReference
 from metricflow.specs import (
     MetricInputMeasureSpec,
     OrderBySpec,
-    InstanceSpec,
     MetricSpec,
     LinklessIdentifierSpec,
     TimeDimensionSpec,
     SpecWhereClauseConstraint,
+    InstanceSpecSet,
 )
+from metricflow.sql.sql_plan import SqlJoinType
 from metricflow.time.time_granularity import TimeGranularity
 from metricflow.visitor import Visitable, VisitorOutputT
 
+logger = logging.getLogger(__name__)
+
 # The type of data set that is flowing out of the source nodes
 SourceDataSetT = TypeVar("SourceDataSetT", bound=DataSet)
+NodeSelfT = TypeVar("NodeSelfT", bound="DataflowPlanNode")
 
 
 class DataflowPlanNode(Generic[SourceDataSetT], DagNode, Visitable, ABC):
@@ -78,7 +84,28 @@ class DataflowPlanNode(Generic[SourceDataSetT], DagNode, Visitable, ABC):
     @abstractmethod
     def accept(self, visitor: DataflowPlanNodeVisitor[SourceDataSetT, VisitorOutputT]) -> VisitorOutputT:
         """Called when a visitor needs to visit this node."""
-        pass
+        raise NotImplementedError
+
+    @abstractmethod
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:
+        """Returns true if this node the same functionality as the other node.
+
+        In other words, this returns true if  all parameters (aside from parent_nodes) are the same.
+        """
+        raise NotImplementedError
+
+    @abstractmethod
+    def with_new_parents(self: NodeSelfT, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]) -> NodeSelfT:
+        """Creates a node with the same behavior as this node, but with a different set of parents
+
+        typing.Self would be useful here, but not available in Python 3.8.
+        """
+        raise NotImplementedError
+
+    @property
+    def node_type(self) -> Type:  # noqa: D
+        # TODO: Remove.
+        return self.__class__
 
 
 class DataflowPlanNodeVisitor(Generic[SourceDataSetT, VisitorOutputT], ABC):
@@ -159,6 +186,10 @@ class DataflowPlanNodeVisitor(Generic[SourceDataSetT, VisitorOutputT], ABC):
     ) -> VisitorOutputT:
         pass
 
+    @abstractmethod
+    def visit_join_to_time_spine_node(self, node: JoinToTimeSpineNode[SourceDataSetT]) -> VisitorOutputT:  # noqa: D
+        pass
+
 
 class BaseOutput(Generic[SourceDataSetT], DataflowPlanNode[SourceDataSetT], ABC):
     """A node that outputs data in a "base" format.
@@ -212,6 +243,23 @@ class ReadSqlSourceNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
             DisplayedProperty("data_set", self.data_set),
         ]
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__) and other_node.data_set == self.data_set
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> ReadSqlSourceNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 0
+        return ReadSqlSourceNode[SourceDataSetT](data_set=self.data_set)
+
+
+@dataclass(frozen=True)
+class ValidityWindowJoinDescription:
+    """Encapsulates details about join constraints around validity windows"""
+
+    window_start_dimension: TimeDimensionSpec
+    window_end_dimension: TimeDimensionSpec
+
 
 @dataclass(frozen=True)
 class JoinDescription(Generic[SourceDataSetT]):
@@ -223,24 +271,26 @@ class JoinDescription(Generic[SourceDataSetT]):
     join_on_partition_dimensions: Tuple[PartitionDimensionJoinDescription, ...]
     join_on_partition_time_dimensions: Tuple[PartitionTimeDimensionJoinDescription, ...]
 
+    validity_window: Optional[ValidityWindowJoinDescription] = None
+
 
 class JoinToBaseOutputNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     """A node that joins data from other nodes to a standard output node, one by one via identifier."""
 
     def __init__(
         self,
-        parent_node: BaseOutput[SourceDataSetT],
+        left_node: BaseOutput[SourceDataSetT],
         join_targets: List[JoinDescription[SourceDataSetT]],
         node_id: Optional[NodeId] = None,
     ) -> None:
         """Constructor.
 
         Args:
-            parent_node: node with standard output
+            left_node: node with standard output
             join_targets: other sources that should be joined to this node.
             node_id: Override the node ID with this value
         """
-        self._left_node = parent_node
+        self._left_node = left_node
         self._join_targets = join_targets
 
         # Doing a list comprehension throws a type error, so doing it this way.
@@ -275,6 +325,44 @@ class JoinToBaseOutputNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
             for i, join_description in enumerate(self._join_targets)
         ]
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        if not isinstance(other_node, self.__class__) or len(self.join_targets) != len(other_node.join_targets):
+            return False
+
+        for i in range(len(self.join_targets)):
+            if (
+                self.join_targets[i].join_on_identifier != other_node.join_targets[i].join_on_identifier
+                or self.join_targets[i].join_on_partition_dimensions
+                != other_node.join_targets[i].join_on_partition_dimensions
+                or self.join_targets[i].join_on_partition_time_dimensions
+                != other_node.join_targets[i].join_on_partition_time_dimensions
+                or self.join_targets[i].validity_window != other_node.join_targets[i].validity_window
+            ):
+                return False
+        return True
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> JoinToBaseOutputNode[SourceDataSetT]:
+        assert len(new_parent_nodes) > 1
+        new_left_node = new_parent_nodes[0]
+        new_join_nodes = new_parent_nodes[1:]
+        assert len(new_join_nodes) == len(self._join_targets)
+
+        return JoinToBaseOutputNode[SourceDataSetT](
+            left_node=new_left_node,
+            join_targets=[
+                JoinDescription(
+                    join_node=new_join_nodes[i],
+                    join_on_identifier=old_join_target.join_on_identifier,
+                    join_on_partition_dimensions=old_join_target.join_on_partition_dimensions,
+                    join_on_partition_time_dimensions=old_join_target.join_on_partition_time_dimensions,
+                    validity_window=old_join_target.validity_window,
+                )
+                for i, old_join_target in enumerate(self._join_targets)
+            ],
+        )
+
 
 class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     """A node that allows for cumulative metric computation by doing a self join across a cumulative date range."""
@@ -282,8 +370,7 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
     def __init__(
         self,
         parent_node: BaseOutput[SourceDataSetT],
-        metric_time_dimension_reference: TimeDimensionReference,
-        window: Optional[CumulativeMetricWindow],
+        window: Optional[MetricTimeWindow],
         grain_to_date: Optional[TimeGranularity],
         node_id: Optional[NodeId] = None,
         time_range_constraint: Optional[TimeRangeConstraint] = None,
@@ -292,17 +379,20 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
 
         Args:
             parent_node: node with standard output
-            metric_time_dimension_reference: the name of the virtual time dimension used in queries to refer to the time
-            dimension that each measure should be aggregated by.
             window: time window to join over
-            grain_to_date: indicates time range should start from the beginning of this time granularity (eg month to day)
+            grain_to_date: indicates time range should start from the beginning of this time granularity
+            (eg month to day)
             node_id: Override the node ID with this value
             time_range_constraint: time range to aggregate over
         """
+        if window and grain_to_date:
+            raise RuntimeError(
+                f"This node cannot be initialized with both window and grain_to_date set. This configuration should "
+                f"have been prevented by model validation. window: {window}. grain_to_date: {grain_to_date}."
+            )
         self._parent_node = parent_node
         self._grain_to_date = grain_to_date
         self._window = window
-        self._metric_time_dimension_reference = metric_time_dimension_reference
         self.time_range_constraint = time_range_constraint
 
         # Doing a list comprehension throws a type error, so doing it this way.
@@ -316,10 +406,6 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
     def accept(self, visitor: DataflowPlanNodeVisitor[SourceDataSetT, VisitorOutputT]) -> VisitorOutputT:  # noqa: D
         return visitor.visit_join_over_time_range_node(self)
 
-    @property
-    def metric_time_dimension_reference(self) -> TimeDimensionReference:  # noqa: D
-        return self._metric_time_dimension_reference
-
     @property
     def grain_to_date(self) -> Optional[TimeGranularity]:  # noqa: D
         return self._grain_to_date
@@ -333,13 +419,32 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
         return self._parent_node
 
     @property
-    def window(self) -> Optional[CumulativeMetricWindow]:  # noqa: D
+    def window(self) -> Optional[MetricTimeWindow]:  # noqa: D
         return self._window
 
     @property
     def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
         return super().displayed_properties
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return (
+            isinstance(other_node, self.__class__)
+            and other_node.grain_to_date == self.grain_to_date
+            and other_node.window == self.window
+            and other_node.time_range_constraint == self.time_range_constraint
+        )
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> JoinOverTimeRangeNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return JoinOverTimeRangeNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            window=self.window,
+            grain_to_date=self.grain_to_date,
+            time_range_constraint=self.time_range_constraint,
+        )
+
 
 class AggregatedMeasuresOutput(Generic[SourceDataSetT], BaseOutput[SourceDataSetT], ABC):
     """A node that outputs data where the measures are aggregated.
@@ -394,6 +499,21 @@ class AggregateMeasuresNode(Generic[SourceDataSetT], AggregatedMeasuresOutput[So
         """
         return self._metric_input_measure_specs
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return (
+            isinstance(other_node, self.__class__)
+            and other_node.metric_input_measure_specs == self.metric_input_measure_specs
+        )
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> AggregateMeasuresNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return AggregateMeasuresNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            metric_input_measure_specs=self.metric_input_measure_specs,
+        )
+
 
 class JoinAggregatedMeasuresByGroupByColumnsNode(Generic[SourceDataSetT], AggregatedMeasuresOutput[SourceDataSetT]):
     """A node that joins aggregated measures with group by elements.
@@ -438,6 +558,16 @@ class JoinAggregatedMeasuresByGroupByColumnsNode(Generic[SourceDataSetT], Aggreg
             DisplayedProperty("Join aggregated measure nodes: ", f"{[node.node_id for node in self.parent_nodes]}")
         ]
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__)
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> JoinAggregatedMeasuresByGroupByColumnsNode[SourceDataSetT]:
+        return JoinAggregatedMeasuresByGroupByColumnsNode[SourceDataSetT](
+            parent_nodes=new_parent_nodes,
+        )
+
 
 class SemiAdditiveJoinNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     """A node that performs a row filter by aggregating a given non-additive dimension.
@@ -550,6 +680,31 @@ class SemiAdditiveJoinNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
         return super().displayed_properties
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        if not isinstance(other_node, self.__class__):
+            return False
+
+        return (
+            isinstance(other_node, self.__class__)
+            and other_node.identifier_specs == self.identifier_specs
+            and other_node.time_dimension_spec == self.time_dimension_spec
+            and other_node.agg_by_function == self.agg_by_function
+            and other_node.queried_time_dimension_spec == self.queried_time_dimension_spec
+        )
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> SemiAdditiveJoinNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+
+        return SemiAdditiveJoinNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            identifier_specs=self.identifier_specs,
+            time_dimension_spec=self.time_dimension_spec,
+            agg_by_function=self.agg_by_function,
+            queried_time_dimension_spec=self.queried_time_dimension_spec,
+        )
+
 
 class ComputedMetricsOutput(Generic[SourceDataSetT], BaseOutput[SourceDataSetT], ABC):
     """A node that outputs data that contains metrics computed from measures."""
@@ -557,6 +712,95 @@ class ComputedMetricsOutput(Generic[SourceDataSetT], BaseOutput[SourceDataSetT],
     pass
 
 
+class JoinToTimeSpineNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT], ABC):
+    """Join parent dataset to time spine dataset."""
+
+    def __init__(
+        self,
+        parent_node: BaseOutput[SourceDataSetT],
+        time_range_constraint: Optional[TimeRangeConstraint] = None,
+        offset_window: Optional[MetricTimeWindow] = None,
+        offset_to_grain: Optional[TimeGranularity] = None,
+    ) -> None:  # noqa: D
+        """Constructor.
+
+        Args:
+            parent_node: Node that returns desired dataset to join to time spine.
+            time_range_constraint: Time range to constrain the time spine to.
+            offset_window: Time window to offset the parent dataset by when joining to time spine.
+            offset_to_grain: Granularity period to offset the parent dataset to when joining to time spine.
+
+        Passing both offset_window and offset_to_grain not allowed.
+        """
+        assert not (
+            offset_window and offset_to_grain
+        ), "Can't set both offset_window and offset_to_grain when joining to time spine. Choose one or the other."
+        self._parent_node = parent_node
+        self._offset_window = offset_window
+        self._offset_to_grain = offset_to_grain
+        self._time_range_constraint = time_range_constraint
+
+        super().__init__(node_id=self.create_unique_id(), parent_nodes=[self._parent_node])
+
+    @classmethod
+    def id_prefix(cls) -> str:  # noqa: D
+        return DATAFLOW_NODE_JOIN_TO_TIME_SPINE_ID_PREFIX
+
+    @property
+    def time_range_constraint(self) -> Optional[TimeRangeConstraint]:  # noqa: D
+        """Time range constraint to apply when querying time spine table."""
+        return self._time_range_constraint
+
+    @property
+    def offset_window(self) -> Optional[MetricTimeWindow]:  # noqa: D
+        """Time range constraint to apply when querying time spine table."""
+        return self._offset_window
+
+    @property
+    def offset_to_grain(self) -> Optional[TimeGranularity]:  # noqa: D
+        """Time range constraint to apply when querying time spine table."""
+        return self._offset_to_grain
+
+    def accept(self, visitor: DataflowPlanNodeVisitor[SourceDataSetT, VisitorOutputT]) -> VisitorOutputT:  # noqa: D
+        return visitor.visit_join_to_time_spine_node(self)
+
+    @property
+    def description(self) -> str:  # noqa: D
+        return """Join to Time Spine Dataset"""
+
+    @property
+    def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
+        return super().displayed_properties + [
+            DisplayedProperty("time_range_constraint", self._time_range_constraint),
+            DisplayedProperty("offset_window", self._offset_window),
+            DisplayedProperty("offset_to_grain", self._offset_to_grain),
+        ]
+
+    @property
+    def parent_node(self) -> BaseOutput:  # noqa: D
+        return self._parent_node
+
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return (
+            isinstance(other_node, self.__class__)
+            and other_node.time_range_constraint == self.time_range_constraint
+            and other_node.offset_window == self.offset_window
+            and other_node.offset_to_grain == self.offset_to_grain
+        )
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> JoinToTimeSpineNode[SourceDataSetT]:
+        # Is this right? how is this method used?
+        assert len(new_parent_nodes) == 1
+        return JoinToTimeSpineNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            time_range_constraint=self.time_range_constraint,
+            offset_window=self._offset_window,
+            offset_to_grain=self._offset_to_grain,
+        )
+
+
 class ComputeMetricsNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceDataSetT]):
     """A node that computes metrics from input measures. Dimensions / identifiers are passed through."""
 
@@ -597,6 +841,24 @@ class ComputeMetricsNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceDa
     def parent_node(self) -> BaseOutput:  # noqa: D
         return self._parent_node
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        if not isinstance(other_node, self.__class__):
+            return False
+
+        if other_node.metric_specs != self.metric_specs:
+            return False
+
+        return isinstance(other_node, self.__class__) and other_node.metric_specs == self.metric_specs
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> ComputeMetricsNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return ComputeMetricsNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            metric_specs=self.metric_specs,
+        )
+
 
 class OrderByLimitNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceDataSetT]):
     """A node that re-orders the input data with a limit."""
@@ -616,9 +878,8 @@ class OrderByLimitNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceData
         """
         self._order_by_specs = order_by_specs
         self._limit = limit
-        self._metrics_output_node = parent_node
         self._parent_node = parent_node
-        super().__init__(node_id=self.create_unique_id(), parent_nodes=[self._metrics_output_node])
+        super().__init__(node_id=self.create_unique_id(), parent_nodes=[self._parent_node])
 
     @classmethod
     def id_prefix(cls) -> str:  # noqa: D
@@ -655,6 +916,24 @@ class OrderByLimitNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceData
     def parent_node(self) -> Union[BaseOutput[SourceDataSetT], ComputedMetricsOutput[SourceDataSetT]]:  # noqa: D
         return self._parent_node
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return (
+            isinstance(other_node, self.__class__)
+            and other_node.order_by_specs == self.order_by_specs
+            and other_node.limit == self.limit
+        )
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> OrderByLimitNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+
+        return OrderByLimitNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            order_by_specs=self.order_by_specs,
+            limit=self.limit,
+        )
+
 
 class MetricTimeDimensionTransformNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     """A node transforms the input data set so that it contains the metric time dimension and relevant measures.
@@ -702,6 +981,21 @@ class MetricTimeDimensionTransformNode(Generic[SourceDataSetT], BaseOutput[Sourc
     def parent_node(self) -> BaseOutput[SourceDataSetT]:  # noqa: D
         return self._parent_node
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return (
+            isinstance(other_node, self.__class__)
+            and other_node.aggregation_time_dimension_reference == self.aggregation_time_dimension_reference
+        )
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> MetricTimeDimensionTransformNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return MetricTimeDimensionTransformNode(
+            parent_node=new_parent_nodes[0],
+            aggregation_time_dimension_reference=self.aggregation_time_dimension_reference,
+        )
+
 
 class SinkNodeVisitor(Generic[SourceDataSetT, VisitorOutputT], ABC):
     """Similar to DataflowPlanNodeVisitor, but only for sink nodes."""
@@ -737,7 +1031,7 @@ class SinkOutput(Generic[SourceDataSetT], DataflowPlanNode[SourceDataSetT], ABC)
 class WriteToResultDataframeNode(Generic[SourceDataSetT], SinkOutput[SourceDataSetT]):
     """A node where incoming data gets written to a dataframe."""
 
-    def __init__(self, parent_node: BaseOutput) -> None:  # noqa: D
+    def __init__(self, parent_node: BaseOutput[SourceDataSetT]) -> None:  # noqa: D
         self._parent_node = parent_node
         super().__init__(node_id=self.create_unique_id(), parent_nodes=[parent_node])
 
@@ -762,6 +1056,15 @@ class WriteToResultDataframeNode(Generic[SourceDataSetT], SinkOutput[SourceDataS
     ) -> VisitorOutputT:
         return visitor.visit_write_to_result_dataframe_node(self)
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__)
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> WriteToResultDataframeNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return WriteToResultDataframeNode[SourceDataSetT](parent_node=new_parent_nodes[0])
+
 
 class WriteToResultTableNode(Generic[SourceDataSetT], SinkOutput[SourceDataSetT]):
     """A node where incoming data gets written to a table."""
@@ -806,6 +1109,17 @@ class WriteToResultTableNode(Generic[SourceDataSetT], SinkOutput[SourceDataSetT]
     def output_sql_table(self) -> SqlTable:  # noqa: D
         return self._output_sql_table
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__) and other_node.output_sql_table == self.output_sql_table
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> WriteToResultTableNode:
+        return WriteToResultTableNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            output_sql_table=self.output_sql_table,
+        )
+
 
 class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     """Only passes the listed elements."""
@@ -813,11 +1127,12 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
     def __init__(  # noqa: D
         self,
         parent_node: BaseOutput[SourceDataSetT],
-        include_specs: Sequence[InstanceSpec],
+        include_specs: InstanceSpecSet,
         replace_description: Optional[str] = None,
     ) -> None:
         self._include_specs = include_specs
         self._replace_description = replace_description
+        self._parent_node = parent_node
         super().__init__(node_id=self.create_unique_id(), parent_nodes=[parent_node])
 
     @classmethod
@@ -825,9 +1140,9 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
         return DATAFLOW_NODE_PASS_FILTER_ELEMENTS_ID_PREFIX
 
     @property
-    def include_specs(self) -> List[InstanceSpec]:
+    def include_specs(self) -> InstanceSpecSet:
         """Returns the specs for the elements that it should pass."""
-        return list(self._include_specs)
+        return self._include_specs
 
     def accept(self, visitor: DataflowPlanNodeVisitor[SourceDataSetT, VisitorOutputT]) -> VisitorOutputT:  # noqa: D
         return visitor.visit_pass_elements_filter_node(self)
@@ -838,7 +1153,7 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
             return self._replace_description
 
         formatted_str = textwrap.indent(
-            pformat_big_objects([x.qualified_name for x in self._include_specs]), prefix="  "
+            pformat_big_objects([x.qualified_name for x in self._include_specs.all_specs]), prefix="  "
         )
         return f"Pass Only Elements:\n{formatted_str}"
 
@@ -847,14 +1162,26 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
         additional_properties = []
         if not self._replace_description:
             additional_properties = [
-                DisplayedProperty("include_spec", include_spec) for include_spec in self._include_specs
+                DisplayedProperty("include_spec", include_spec) for include_spec in self._include_specs.all_specs
             ]
         return super().displayed_properties + additional_properties
 
     @property
-    def parent_node(self) -> DataflowPlanNode:  # noqa: D
-        assert len(self.parent_nodes) == 1
-        return self.parent_nodes[0]
+    def parent_node(self) -> BaseOutput[SourceDataSetT]:  # noqa: D
+        return self._parent_node
+
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__) and other_node.include_specs == self.include_specs
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> FilterElementsNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return FilterElementsNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            include_specs=self.include_specs,
+            replace_description=self._replace_description,
+        )
 
 
 class WhereConstraintNode(AggregatedMeasuresOutput[SourceDataSetT]):
@@ -891,14 +1218,28 @@ class WhereConstraintNode(AggregatedMeasuresOutput[SourceDataSetT]):
     def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
         return super().displayed_properties + [DisplayedProperty("where_condition", self.where)]
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__) and other_node.where == self.where
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> WhereConstraintNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return WhereConstraintNode(
+            parent_node=new_parent_nodes[0],
+            where_constraint=self.where,
+        )
+
 
 class CombineMetricsNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceDataSetT]):
     """Combines metrics from different nodes into a single output"""
 
     def __init__(  # noqa: D
         self,
-        parent_nodes: Sequence[ComputedMetricsOutput[SourceDataSetT]],
+        parent_nodes: Sequence[Union[BaseOutput, ComputedMetricsOutput[SourceDataSetT]]],
+        join_type: SqlJoinType = SqlJoinType.FULL_OUTER,
     ) -> None:
+        self._join_type = join_type
         super().__init__(node_id=self.create_unique_id(), parent_nodes=list(parent_nodes))
 
     @classmethod
@@ -912,6 +1253,34 @@ class CombineMetricsNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceDa
     def description(self) -> str:  # noqa: D
         return "Combine Metrics"
 
+    @property
+    def displayed_properties(self) -> List[DisplayedProperty]:
+        """Prints details about the join types and how the node will behave"""
+        custom_properties = [DisplayedProperty("join type", self.join_type)]
+        if self.join_type is SqlJoinType.FULL_OUTER:
+            custom_properties.append(
+                DisplayedProperty("de-duplication method", "post-join aggregation across all dimensions")
+            )
+
+        return super().displayed_properties + custom_properties
+
+    @property
+    def join_type(self) -> SqlJoinType:
+        """The type of join used for combining metrics."""
+        return self._join_type
+
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__) and other_node.join_type == self.join_type
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> CombineMetricsNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return CombineMetricsNode(
+            parent_nodes=new_parent_nodes,
+            join_type=self.join_type,
+        )
+
 
 class ConstrainTimeRangeNode(AggregatedMeasuresOutput[SourceDataSetT], BaseOutput[SourceDataSetT]):
     """Constrains the time range of the input data set.
@@ -958,6 +1327,18 @@ class ConstrainTimeRangeNode(AggregatedMeasuresOutput[SourceDataSetT], BaseOutpu
             DisplayedProperty("time_range_end", self.time_range_constraint.end_time.isoformat()),
         ]
 
+    def functionally_identical(self, other_node: DataflowPlanNode[SourceDataSetT]) -> bool:  # noqa: D
+        return isinstance(other_node, self.__class__) and self.time_range_constraint == other_node.time_range_constraint
+
+    def with_new_parents(  # noqa: D
+        self, new_parent_nodes: Sequence[BaseOutput[SourceDataSetT]]
+    ) -> ConstrainTimeRangeNode[SourceDataSetT]:
+        assert len(new_parent_nodes) == 1
+        return ConstrainTimeRangeNode[SourceDataSetT](
+            parent_node=new_parent_nodes[0],
+            time_range_constraint=self.time_range_constraint,
+        )
+
 
 class DataflowPlan(Generic[SourceDataSetT], MetricFlowDag[SinkOutput[SourceDataSetT]]):
     """Describes the flow of metric data as it goes from source nodes to sink nodes in the graph."""
@@ -974,5 +1355,5 @@ class DataflowPlan(Generic[SourceDataSetT], MetricFlowDag[SinkOutput[SourceDataS
 
     @property
     def sink_output_node(self) -> SinkOutput[SourceDataSetT]:  # noqa: D
-        assert len(self._sink_output_nodes) == 1
+        assert len(self._sink_output_nodes) == 1, f"Only 1 sink node supported. Got: {self._sink_output_nodes}"
         return self._sink_output_nodes[0]
