@@ -3,7 +3,8 @@ from __future__ import annotations
 from typing import List, Optional
 
 from metricflow.dataflow.sql_table import SqlTable
-from metricflow.model.objects.utils import ParseableObject, HashableBaseModel
+from metricflow.model.objects.common import Metadata
+from metricflow.model.objects.base import ModelWithMetadataParsing, HashableBaseModel
 from metricflow.object_utils import ExtendedEnum
 
 
@@ -21,13 +22,13 @@ class MaterializationFormat(ExtendedEnum):
     WIDE = "wide"
 
 
-class MaterializationTableauParams(HashableBaseModel, ParseableObject):
+class MaterializationTableauParams(HashableBaseModel):
     """Describes the projects to write to in Tableau."""
 
     projects: List[str]
 
 
-class MaterializationDestination(HashableBaseModel, ParseableObject):
+class MaterializationDestination(HashableBaseModel):
     """Describes where/how a materialized table should be written"""
 
     location: MaterializationLocation
@@ -36,11 +37,13 @@ class MaterializationDestination(HashableBaseModel, ParseableObject):
     tableau_params: Optional[MaterializationTableauParams]
 
 
-class Materialization(HashableBaseModel, ParseableObject):
+class Materialization(HashableBaseModel, ModelWithMetadataParsing):
     """Describes a materialization"""
 
     name: str
+    description: Optional[str]
     metrics: List[str]
     dimensions: List[str]
     destinations: Optional[List[MaterializationDestination]]
     destination_table: Optional[SqlTable]
+    metadata: Optional[Metadata]
