@@ -11,7 +11,7 @@ from metricflow.model.objects.base import (
     PydanticCustomInputParser,
     PydanticParseableValueType,
 )
-from metricflow.object_utils import ExtendedEnum, hash_strings
+from metricflow.object_utils import ExtendedEnum, hash_items
 from metricflow.references import MeasureReference
 from metricflow.time.time_granularity import TimeGranularity
 from metricflow.time.time_granularity import string_to_time_granularity
@@ -24,6 +24,7 @@ class MetricType(ExtendedEnum):
     RATIO = "ratio"
     EXPR = "expr"
     CUMULATIVE = "cumulative"
+    DERIVED = "derived"
 
 
 class MetricInputMeasure(PydanticCustomInputParser, HashableBaseModel):
@@ -64,8 +65,8 @@ class MetricInputMeasure(PydanticCustomInputParser, HashableBaseModel):
         return MeasureReference(element_name=self.alias or self.name)
 
 
-class CumulativeMetricWindow(PydanticCustomInputParser, HashableBaseModel):
-    """Describes the window of time the metric should be accumulated over. ie '1 day', '2 weeks', etc"""
+class MetricTimeWindow(PydanticCustomInputParser, HashableBaseModel):
+    """Describes the window of time the metric should be accumulated over, e.g., '1 day', '2 weeks', etc"""
 
     count: int
     granularity: TimeGranularity
@@ -74,21 +75,21 @@ class CumulativeMetricWindow(PydanticCustomInputParser, HashableBaseModel):
         return f"{self.count} {self.granularity.value}"
 
     @classmethod
-    def _from_yaml_value(cls, input: PydanticParseableValueType) -> CumulativeMetricWindow:
-        """Parses a CumulativeMetricWindow from a string input found in a user provided model specification
+    def _from_yaml_value(cls, input: PydanticParseableValueType) -> MetricTimeWindow:
+        """Parses a MetricTimeWindow from a string input found in a user provided model specification
 
-        The CumulativeMetricWindow is always expected to be provided as a string in user-defined YAML configs.
+        The MetricTimeWindow is always expected to be provided as a string in user-defined YAML configs.
         """
         if isinstance(input, str):
-            return CumulativeMetricWindow.parse(input)
+            return MetricTimeWindow.parse(input)
         else:
             raise ValueError(
-                f"CumulativeMetricWindow inputs from model configs are expected to always be of type string, but got "
+                f"MetricTimeWindow inputs from model configs are expected to always be of type string, but got "
                 f"type {type(input)} with value: {input}"
             )
 
     @staticmethod
-    def parse(window: str) -> CumulativeMetricWindow:
+    def parse(window: str) -> MetricTimeWindow:
         """Returns window values if parsing succeeds, None otherwise
 
         Output of the form: (<time unit count>, <time granularity>, <error message>) - error message is None if window is formatted properly
@@ -96,7 +97,7 @@ class CumulativeMetricWindow(PydanticCustomInputParser, HashableBaseModel):
         parts = window.split(" ")
         if len(parts) != 2:
             raise ParsingException(
-                f"Invalid window ({window}) in cumulative metric. Should be of the form `<count> <granularity>`, ie `28 days`",
+                f"Invalid window ({window}) in cumulative metric. Should be of the form `<count> <granularity>`, e.g., `28 days`",
             )
 
         granularity = parts[1]
@@ -113,12 +114,22 @@ class CumulativeMetricWindow(PydanticCustomInputParser, HashableBaseModel):
         if not count.isdigit():
             raise ParsingException(f"Invalid count ({count}) in cumulative metric window string: ({window})")
 
-        return CumulativeMetricWindow(
+        return MetricTimeWindow(
             count=int(count),
             granularity=string_to_time_granularity(granularity),
         )
 
 
+class MetricInput(HashableBaseModel):
+    """Provides a pointer to a metric along with the additional properties used on that metric."""
+
+    name: str
+    constraint: Optional[WhereClauseConstraint]
+    alias: Optional[str]
+    offset_window: Optional[MetricTimeWindow]
+    offset_to_grain: Optional[TimeGranularity]
+
+
 class MetricTypeParams(HashableBaseModel):
     """Type params add additional context to certain metric types (the context depends on the metric type)"""
 
@@ -127,8 +138,9 @@ class MetricTypeParams(HashableBaseModel):
     numerator: Optional[MetricInputMeasure]
     denominator: Optional[MetricInputMeasure]
     expr: Optional[str]
-    window: Optional[CumulativeMetricWindow]
+    window: Optional[MetricTimeWindow]
     grain_to_date: Optional[TimeGranularity]
+    metrics: Optional[List[MetricInput]]
 
     @property
     def numerator_measure_reference(self) -> Optional[MeasureReference]:
@@ -170,6 +182,11 @@ class Metric(HashableBaseModel, ModelWithMetadataParsing):
         """Return the measure references associated with all input measure configurations for this metric"""
         return [x.measure_reference for x in self.input_measures]
 
+    @property
+    def input_metrics(self) -> List[MetricInput]:
+        """Return the associated input metrics for this metric"""
+        return self.type_params.metrics or []
+
     @property
     def definition_hash(self) -> str:  # noqa: D
         values: List[str] = [self.name, self.type_params.expr or ""]
@@ -178,4 +195,4 @@ class Metric(HashableBaseModel, ModelWithMetadataParsing):
             if self.constraint.linkable_names:
                 values.extend(self.constraint.linkable_names)
         values.extend([m.element_name for m in self.measure_references])
-        return hash_strings(values)
+        return hash_items(values)
