@@ -376,22 +376,128 @@ class CountAggregationExprRule(ModelValidationRule):
 
         for data_source in model.data_sources:
             for measure in data_source.measures:
+                context = DataSourceElementContext(
+                    file_context=FileContext.from_metadata(metadata=data_source.metadata),
+                    data_source_element=DataSourceElementReference(
+                        data_source_name=data_source.name, element_name=measure.name
+                    ),
+                    element_type=DataSourceElementType.MEASURE,
+                )
                 if measure.agg == AggregationType.COUNT and measure.expr is None:
                     issues.append(
                         ValidationError(
-                            context=DataSourceElementContext(
-                                file_context=FileContext.from_metadata(metadata=data_source.metadata),
-                                data_source_element=DataSourceElementReference(
-                                    data_source_name=data_source.name, element_name=measure.name
-                                ),
-                                element_type=DataSourceElementType.MEASURE,
-                            ),
+                            context=context,
                             message=(
                                 f"Measure '{measure.name}' uses a COUNT aggregation, which requires an expr to be provided. "
                                 f"Provide 'expr: 1' if a count of all rows is desired."
                             ),
                         )
                     )
+                if (
+                    measure.agg == AggregationType.COUNT
+                    and measure.expr
+                    and measure.expr.lower().startswith("distinct ")
+                ):
+                    # TODO: Expand this to include SUM and potentially AVG agg types as well
+                    # Note expansion of this guard requires the addition of sum_distinct and avg_distinct agg types
+                    # or else an adjustment to the error message below.
+                    issues.append(
+                        ValidationError(
+                            context=context,
+                            message=(
+                                f"Measure '{measure.name}' uses a '{measure.agg.value}' aggregation with a DISTINCT expr: "
+                                f"'{measure.expr}. This is not supported, as it effectively converts an additive "
+                                f"measure into a non-additive one, and this could cause certain queries to return "
+                                f"incorrect results. Please use the {measure.agg.value}_distinct aggregation type."
+                            ),
+                        )
+                    )
+        return issues
+
+
+class PercentileAggregationRule(ModelValidationRule):
+    """Checks that only PERCENTILE measures have agg_params and valid percentile value provided."""
+
+    @staticmethod
+    @validate_safely(
+        whats_being_done="running model validation ensuring the agg_params.percentile value exist for measures with percentile aggregation"
+    )
+    def validate_model(model: UserConfiguredModel) -> List[ValidationIssueType]:  # noqa: D
+        issues: List[ValidationIssueType] = []
+
+        for data_source in model.data_sources:
+            for measure in data_source.measures:
+                context = DataSourceElementContext(
+                    file_context=FileContext.from_metadata(metadata=data_source.metadata),
+                    data_source_element=DataSourceElementReference(
+                        data_source_name=data_source.name, element_name=measure.name
+                    ),
+                    element_type=DataSourceElementType.MEASURE,
+                )
+                if measure.agg == AggregationType.PERCENTILE:
+                    if measure.agg_params is None or measure.agg_params.percentile is None:
+                        issues.append(
+                            ValidationError(
+                                context=context,
+                                message=(
+                                    f"Measure '{measure.name}' uses a PERCENTILE aggregation, which requires agg_params.percentile to be provided."
+                                ),
+                            )
+                        )
+                    elif measure.agg_params.percentile <= 0 or measure.agg_params.percentile >= 1:
+                        issues.append(
+                            ValidationError(
+                                context=context,
+                                message=(
+                                    f"Percentile aggregation parameter for measure '{measure.name}' is '{measure.agg_params.percentile}', but"
+                                    "must be between 0 and 1 (non-inclusive). For example, to indicate the 65th percentile value, set 'percentile: 0.65'. "
+                                    "For percentile values of 0, please use MIN, for percentile values of 1, please use MAX."
+                                ),
+                            )
+                        )
+                elif measure.agg == AggregationType.MEDIAN:
+                    if measure.agg_params:
+                        if measure.agg_params.percentile is not None and measure.agg_params.percentile != 0.5:
+                            issues.append(
+                                ValidationError(
+                                    context=context,
+                                    message=f"Measure '{measure.name}' uses a MEDIAN aggregation, while percentile is set to "
+                                    f"'{measure.agg_params.percentile}', a conflicting value. Please remove the parameter "
+                                    "or set to '0.5'.",
+                                )
+                            )
+                        if measure.agg_params.use_discrete_percentile:
+                            issues.append(
+                                ValidationError(
+                                    context=context,
+                                    message=f"Measure '{measure.name}' uses a MEDIAN aggregation, while use_discrete_percentile"
+                                    f"is set to true. Please remove the parameter or set to False.",
+                                )
+                            )
+                elif measure.agg_params and (
+                    measure.agg_params.percentile
+                    or measure.agg_params.use_discrete_percentile
+                    or measure.agg_params.use_approximate_percentile
+                ):
+                    wrong_params = []
+                    if measure.agg_params.percentile:
+                        wrong_params.append("percentile")
+                    if measure.agg_params.use_discrete_percentile:
+                        wrong_params.append("use_discrete_percentile")
+                    if measure.agg_params.use_approximate_percentile:
+                        wrong_params.append("use_approximate_percentile")
+
+                    wrong_params_str = ", ".join(wrong_params)
+
+                    issues.append(
+                        ValidationError(
+                            context=context,
+                            message=(
+                                f"Measure '{measure.name}' with aggregation '{measure.agg.value}' uses agg_params "
+                                f"({wrong_params_str}) only relevant to Percentile measures."
+                            ),
+                        )
+                    )
         return issues
 
 
