@@ -1,48 +1,66 @@
 from __future__ import annotations
 
+import json
 import logging
 import threading
 import urllib.parse
+from collections import OrderedDict
 from contextlib import contextmanager
-from typing import ClassVar, Optional, Dict, Iterator, List, Tuple, Any
+from typing import ClassVar, Optional, Dict, Iterator, List, Tuple, Any, Set, Sequence, Callable
 
 import pandas as pd
 import sqlalchemy
 from sqlalchemy.exc import ProgrammingError
 
-from metricflow.protocols.sql_client import SupportedSqlEngine, SqlEngineAttributes
-from metricflow.sql.render.sql_plan_renderer import DefaultSqlQueryPlanRenderer
+from metricflow.sql.render.snowflake import SnowflakeSqlQueryPlanRenderer
+from metricflow.protocols.sql_client import SqlEngine, SqlIsolationLevel
+from metricflow.protocols.sql_client import SqlEngineAttributes
+from metricflow.protocols.sql_request import (
+    SqlRequestTagSet,
+    JsonDict,
+    MF_SYSTEM_TAGS_KEY,
+    MF_EXTRA_TAGS_KEY,
+    SqlJsonTag,
+)
 from metricflow.sql.render.sql_plan_renderer import SqlQueryPlanRenderer
 from metricflow.sql.sql_bind_parameters import SqlBindParameters
-from metricflow.sql_clients.common_client import SqlDialect, not_empty
+from metricflow.sql_clients.async_request import SqlStatementCommentMetadata, CombinedSqlTags
+from metricflow.sql_clients.common_client import SqlDialect, not_empty, check_isolation_level
 from metricflow.sql_clients.sqlalchemy_dialect import SqlAlchemySqlClient
 
 
+logger = logging.getLogger(__name__)
+
+
 class SnowflakeEngineAttributes:
     """Engine-specific attributes for the Snowflake query engine
 
     This is an implementation of the SqlEngineAttributes protocol for Snowflake
     """
 
-    sql_engine_type: ClassVar[SupportedSqlEngine] = SupportedSqlEngine.SNOWFLAKE
+    sql_engine_type: ClassVar[SqlEngine] = SqlEngine.SNOWFLAKE
 
     # SQL Engine capabilities
+    supported_isolation_levels: ClassVar[Sequence[SqlIsolationLevel]] = ()
     date_trunc_supported: ClassVar[bool] = True
     full_outer_joins_supported: ClassVar[bool] = True
     indexes_supported: ClassVar[bool] = False
     multi_threading_supported: ClassVar[bool] = True
     timestamp_type_supported: ClassVar[bool] = True
     timestamp_to_string_comparison_supported: ClassVar[bool] = True
+    cancel_submitted_queries_supported: ClassVar[bool] = True
+    continuous_percentile_aggregation_supported: ClassVar[bool] = True
+    discrete_percentile_aggregation_supported: ClassVar[bool] = True
+    approximate_continuous_percentile_aggregation_supported: ClassVar[bool] = True
+    approximate_discrete_percentile_aggregation_supported: ClassVar[bool] = False
 
     # SQL Dialect replacement strings
     double_data_type_name: ClassVar[str] = "DOUBLE"
     timestamp_type_name: ClassVar[Optional[str]] = "TIMESTAMP"
+    random_function_name: ClassVar[str] = "RANDOM"
 
     # MetricFlow attributes
-    sql_query_plan_renderer: ClassVar[SqlQueryPlanRenderer] = DefaultSqlQueryPlanRenderer()
-
-
-logger = logging.getLogger(__name__)
+    sql_query_plan_renderer: ClassVar[SqlQueryPlanRenderer] = SnowflakeSqlQueryPlanRenderer()
 
 
 class SnowflakeSqlClient(SqlAlchemySqlClient):
@@ -129,6 +147,8 @@ class SnowflakeSqlClient(SqlAlchemySqlClient):
             query=url_query_params,
         )
         self._engine_lock = threading.Lock()
+        self._known_sessions_ids_lock = threading.Lock()
+        self._known_session_ids: Set[int] = set()
         super().__init__(
             engine=self._create_engine(login_timeout=login_timeout, client_session_keep_alive=client_session_keep_alive)
         )
@@ -152,7 +172,13 @@ class SnowflakeSqlClient(SqlAlchemySqlClient):
         return SnowflakeEngineAttributes()
 
     @contextmanager
-    def engine_connection(self, engine: sqlalchemy.engine.Engine) -> Iterator[sqlalchemy.engine.Connection]:
+    def _engine_connection(
+        self,
+        engine: sqlalchemy.engine.Engine,
+        isolation_level: Optional[SqlIsolationLevel] = None,
+        system_tags: SqlRequestTagSet = SqlRequestTagSet(),
+        extra_tags: SqlJsonTag = SqlJsonTag(),
+    ) -> Iterator[sqlalchemy.engine.Connection]:
         """Context Manager for providing a configured connection.
 
         Snowflake allows setting a WEEK_START parameter on each session. This forces the value to be
@@ -160,21 +186,46 @@ class SnowflakeSqlClient(SqlAlchemySqlClient):
         options construct, which the DBClient could read in at initialization and use here (for example).
         At this time we hard-code the ISO standard.
         """
-        conn = engine.connect()
-        try:
+        check_isolation_level(self, isolation_level)
+        with super()._engine_connection(self._engine, isolation_level=isolation_level) as conn:
             # WEEK_START 1 means Monday.
             conn.execute("ALTER SESSION SET WEEK_START = 1;")
+            combined_tags: JsonDict = OrderedDict()
+            if system_tags.tag_dict:
+                combined_tags[MF_SYSTEM_TAGS_KEY] = system_tags.tag_dict
+            if extra_tags is not None:
+                combined_tags[MF_EXTRA_TAGS_KEY] = extra_tags.json_dict
+
+            if combined_tags:
+                conn.execute(
+                    sqlalchemy.text("ALTER SESSION SET QUERY_TAG = :query_tag"),
+                    query_tag=json.dumps(combined_tags),
+                )
+            results = conn.execute("SELECT CURRENT_SESSION()")
+            sessions = []
+            for row in results:
+                sessions.append(row[0])
+            assert len(sessions) == 1
+            session = sessions[0]
+            with self._known_sessions_ids_lock:
+                self._known_session_ids.add(session)
             yield conn
-        finally:
-            conn.close()
+            with self._known_sessions_ids_lock:
+                self._known_session_ids.remove(session)
 
     def _query(  # noqa: D
         self,
         stmt: str,
         bind_params: SqlBindParameters = SqlBindParameters(),
+        isolation_level: Optional[SqlIsolationLevel] = None,
         allow_re_auth: bool = True,
+        system_tags: SqlRequestTagSet = SqlRequestTagSet(),
+        extra_tags: SqlJsonTag = SqlJsonTag(),
     ) -> pd.DataFrame:
-        with self.engine_connection(engine=self._engine) as conn:
+        check_isolation_level(self, isolation_level)
+        with self._engine_connection(
+            engine=self._engine, isolation_level=isolation_level, system_tags=system_tags, extra_tags=extra_tags
+        ) as conn:
             try:
                 return pd.read_sql_query(sqlalchemy.text(stmt), conn, params=bind_params.param_dict)
             except ProgrammingError as e:
@@ -186,15 +237,28 @@ class SnowflakeSqlClient(SqlAlchemySqlClient):
                         self._engine.dispose()
                         self._engine = self._create_engine()
                     # this was our one chance to re-auth
-                    return self._query(stmt, allow_re_auth=False, bind_params=bind_params)
+                    return self._query(
+                        stmt, allow_re_auth=False, bind_params=bind_params, isolation_level=isolation_level
+                    )
                 raise e
 
-    def _engine_specific_query_implementation(  # noqa: D
-        self, stmt: str, bind_params: SqlBindParameters
+    def _engine_specific_query_implementation(
+        self,
+        stmt: str,
+        bind_params: SqlBindParameters,
+        isolation_level: Optional[SqlIsolationLevel] = None,
+        system_tags: SqlRequestTagSet = SqlRequestTagSet(),
+        extra_tags: SqlJsonTag = SqlJsonTag(),
     ) -> pd.DataFrame:
-        return self._query(stmt, bind_params)
+        return self._query(
+            stmt,
+            bind_params=bind_params,
+            isolation_level=isolation_level,
+            system_tags=system_tags,
+            extra_tags=extra_tags,
+        )
 
-    def list_tables(self, schema_name: str) -> List[str]:  # noqa: D
+    def list_tables(self, schema_name: str) -> Sequence[str]:  # noqa: D
         df = self.query(
             f"SHOW TABLES IN {schema_name}",
         )
@@ -221,3 +285,35 @@ class SnowflakeSqlClient(SqlAlchemySqlClient):
         """Snowflake will hang pytest if this is not done."""
         with self._engine_lock:
             self._engine.dispose()
+
+    def cancel_submitted_queries(self) -> None:  # noqa: D
+        with super()._engine_connection(self._engine) as conn:
+            with self._known_sessions_ids_lock:
+                for session_id in self._known_session_ids:
+                    logger.info(f"Cancelling queries associated with session id: {session_id}")
+                    conn.execute(f"SELECT SYSTEM$cancel_all_queries({session_id})")
+
+    def cancel_request(self, match_function: Callable[[CombinedSqlTags], bool]) -> int:  # noqa: D
+        # Running queries have an end_time set to the epoch time:
+        # https://docs.snowflake.com/en/sql-reference/functions/query_history.html
+        # Using '1970-01-01' to avoid timezone issues.
+        result = self.query(
+            """
+            SELECT query_id, query_text
+            FROM TABLE(INFORMATION_SCHEMA.QUERY_HISTORY())
+            WHERE end_time <= '1971-01-01'
+            ORDER BY start_time
+            LIMIT 100
+            """
+        )
+        num_cancelled_queries = 0
+        logger.info(f"Found {len(result.values)} queries to examine for cancelling")
+        for query_id, query_text in result.values:
+            parsed_tags = SqlStatementCommentMetadata.parse_tag_metadata_in_comments(query_text)
+            logger.info(f"Tags for {query_id} are: {parsed_tags}")
+            if match_function(parsed_tags):
+                logger.info(f"Cancelling query ID: {query_id}")
+                self.execute(f"SELECT SYSTEM$CANCEL_QUERY('{query_id}')")
+                num_cancelled_queries += 1
+
+        return num_cancelled_queries
