@@ -37,27 +37,72 @@ def create_simple_model_tables(mf_test_session_state: MetricFlowTestSessionState
         # While datetime columns can be used here, somewhere there is a conversion to a string when testing
         # with sqlite, so when you read the datetime column, it comes out as a string.
         # Columns for reference
-        # ["guest_id", "host_id", "listing_id", "booking_value", "is_instant", DEFAULT_DS, "ds_partitioned"]
-        ("u0004114", "u0003141", "l3141592", 742.42, False, "2020-01-01", "2020-01-01"),
-        ("u0004114", "u0003141", "l3141592", 441.14, True, "2020-01-01", "2020-01-01"),
-        ("u0003141", "u0003154", "l2718281", 499.99, True, "2020-01-01", "2020-01-01"),
-        ("u0003154", "u0003141", "l2718281", 719.89, False, "2020-01-01", "2020-01-01"),
-        ("u0004114", "u0003141", "l3141592", 319.85, True, "2020-01-01", "2020-01-01"),
-        ("u0004114", "u0003141", "l2718281", 442.42, True, "2020-01-02", "2020-01-02"),
-        ("u0004114", "u0003141", "l2718281", 241.14, True, "2020-01-02", "2020-01-02"),
-        ("u0004114", "u0003141", "l3141592", 799.99, False, "2020-01-02", "2020-01-02"),
-        ("u0005432", "u0003452", "l2718281", 319.85, True, "2020-01-02", "2020-01-02"),
-        ("u0003452", "u0005432", "l3141592", 519.89, False, "2020-01-02", "2020-01-02"),
-        ("u0003452", "u0004114", "l5948301", 332.23, False, "2020-01-02", "2020-01-02"),
-        ("u0003452", "u0004114", "l5948301", 0.0, False, "2020-01-03", "2020-01-03"),
+        # ["guest_id", "host_id", "listing_id", "booking_value", "is_instant", "ds", "ds_partitioned",
+        #  "booking_paid_at", "referrer_id"]
+        ("u0004114", "u0003141", "l3141592", 742.42, False, "2020-01-01", "2020-01-01", "2020-01-02", None),
+        ("u0004114", "u0003141", "l3141592", 441.14, True, "2020-01-01", "2020-01-01", "2020-01-02", "u0003141"),
+        ("u0003141", "u0003154", "l2718281", 499.99, True, "2020-01-01", "2020-01-01", "2020-01-02", None),
+        ("u0003154", "u0003141", "l2718281", 719.89, False, "2020-01-01", "2020-01-01", "2020-01-02", None),
+        ("u0004114", "u0003141", "l3141592", 319.85, True, "2020-01-01", "2020-01-01", "2020-01-02", "u0005432"),
+        ("u0004114", "u0003141", "l2718281", 442.42, True, "2020-01-02", "2020-01-02", "2020-01-03", None),
+        ("u0004114", "u0003141", "l2718281", 241.14, True, "2020-01-02", "2020-01-02", "2020-01-03", None),
+        ("u0004114", "u0003141", "l3141592", 799.99, False, "2020-01-02", "2020-01-02", "2020-01-03", None),
+        ("u0005432", "u0003452", "l2718281", 319.85, True, "2020-01-02", "2020-01-02", "2020-01-03", "u0003452"),
+        ("u0004114", "u1004114", "l9658588-incomplete", 0.0, False, "2020-01-02", "2020-01-02", "2020-01-03", None),
+        ("u0004114", "u1004114", "l8912456-incomplete", 0.0, False, "2020-01-02", "2020-01-02", "2020-01-03", None),
+        ("u0003452", "u0005432", "l3141592", 519.89, False, "2020-01-02", "2020-01-02", "2020-01-03", "u0003141"),
+        ("u0003452", "u0004114", "l5948301", 332.23, False, "2020-01-02", "2020-01-02", "2020-01-03", "u0003141"),
+        ("u1003452", "u1004114", "no_such_listing", 0.0, False, "2020-01-02", "2020-01-02", "2020-01-03", None),
+        ("u0003452", "u0004114", "l5948301", 0.0, False, "2020-01-03", "2020-01-03", "2020-01-04", None),
+        ("u0004114", "u0003141", "l3141592", 797.42, False, "2019-12-18", "2019-12-18", "2019-12-19", None),
+        ("u0004114", "u0003141", "l3141592", 808.14, True, "2019-12-18", "2019-12-18", "2019-12-19", "u0003141"),
+        ("u0003141", "u0003154", "l2718281", 531.99, True, "2019-12-18", "2019-12-18", "2019-12-19", None),
+        ("u0003154", "u0003141", "l2718281", 285.89, False, "2019-12-18", "2019-12-18", "2019-12-19", None),
+        ("u0004114", "u0003141", "l3141592", 714.85, True, "2019-12-18", "2019-12-18", "2019-12-19", "u0005432"),
+        ("u0004114", "u0003141", "l2718281", 368.42, True, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0004114", "u0003141", "l2718281", 521.14, True, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0004114", "u0003141", "l3141592", 581.99, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0005432", "u0003452", "l2718281", 753.85, True, "2019-12-19", "2019-12-19", "2019-12-20", "u0003452"),
+        ("u0004114", "u1004114", "l9658588-incomplete", 0.0, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0004114", "u1004114", "l8912456-incomplete", 0.0, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0003452", "u0005432", "l3141592", 754.89, False, "2019-12-19", "2019-12-19", "2019-12-20", "u0003141"),
+        ("u0003452", "u0004114", "l5948301", 152.23, False, "2019-12-19", "2019-12-19", "2019-12-20", "u0003141"),
+        ("u1003452", "u1004114", "no_such_listing", 0.0, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0003452", "u0004114", "l5948301", 0.0, False, "2019-12-20", "2019-12-20", "2019-12-21", None),
+        ("u0004114", "u0003141", "l3141592", 352.42, False, "2019-12-18", "2019-12-18", "2019-12-19", None),
+        ("u0004114", "u0003141", "l3141592", 444.14, True, "2019-12-18", "2019-12-18", "2019-12-19", "u0003141"),
+        ("u0003141", "u0003154", "l2718281", 415.99, True, "2019-12-18", "2019-12-18", "2019-12-19", None),
+        ("u0003154", "u0003141", "l2718281", 578.89, False, "2019-12-18", "2019-12-18", "2019-12-19", None),
+        ("u0004114", "u0003141", "l3141592", 768.85, True, "2019-12-18", "2019-12-18", "2019-12-19", "u0005432"),
+        ("u0004114", "u0003141", "l2718281", 532.42, True, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0004114", "u0003141", "l2718281", 258.14, True, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0004114", "u0003141", "l3141592", 753.99, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0005432", "u0003452", "l2718281", 852.85, True, "2019-12-19", "2019-12-19", "2019-12-20", "u0003452"),
+        ("u0004114", "u1004114", "l9658588-incomplete", 0.0, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0004114", "u1004114", "l8912456-incomplete", 0.0, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0003452", "u0005432", "l3141592", 728.89, False, "2019-12-19", "2019-12-19", "2019-12-20", "u0003141"),
+        ("u0003452", "u0004114", "l5948301", 951.23, False, "2019-12-19", "2019-12-19", "2019-12-20", "u0003141"),
+        ("u1003452", "u1004114", "no_such_listing", 0.0, False, "2019-12-19", "2019-12-19", "2019-12-20", None),
+        ("u0003452", "u0004114", "l5948301", 0.0, False, "2019-12-20", "2019-12-20", "2019-12-21", None),
+        ("u0003452", "u0004114", "l5948301", 951.23, False, "2019-12-01", "2019-12-01", "2019-12-01", "u0003141"),
     ]
     create_table(
         sql_client=sql_client,
         sql_table=SqlTable(schema_name=schema, table_name="fct_bookings"),
         df=make_df(
             sql_client=sql_client,
-            columns=["guest_id", "host_id", "listing_id", "booking_value", "is_instant", DEFAULT_DS, "ds_partitioned"],
-            time_columns={DEFAULT_DS, "ds_partitioned"},
+            columns=[
+                "guest_id",
+                "host_id",
+                "listing_id",
+                "booking_value",
+                "is_instant",
+                DEFAULT_DS,
+                "ds_partitioned",
+                "booking_paid_at",
+                "referrer_id",
+            ],
+            time_columns={DEFAULT_DS, "ds_partitioned", "booking_paid_at"},
             data=data,
         ),
     )
@@ -69,7 +114,17 @@ def create_simple_model_tables(mf_test_session_state: MetricFlowTestSessionState
         sql_table=SqlTable(schema_name=schema, table_name="fct_bookings_dt"),
         df=make_df(
             sql_client=sql_client,
-            columns=["guest_id", "host_id", "listing_id", "booking_value", "is_instant", "dt", "dt_partitioned"],
+            columns=[
+                "guest_id",
+                "host_id",
+                "listing_id",
+                "booking_value",
+                "is_instant",
+                "dt",
+                "dt_partitioned",
+                "booking_paid_at",
+                "referrer_id",
+            ],
             time_columns={"dt", "dt_partitioned"},
             data=data,
         ),
@@ -88,6 +143,8 @@ def create_simple_model_tables(mf_test_session_state: MetricFlowTestSessionState
                 ("u0004114", "l2718281", "2020-01-02", "2020-01-02"),
                 ("u0004114", "l3141592", "2020-01-02", "2020-01-02"),
                 ("u1612112", "l2718281", "2020-01-02", "2020-01-02"),
+                ("u0004114", "", "2020-01-02", "2020-01-02"),
+                ("u0004114", "l7891283-incomplete", "2020-01-02", "2020-01-02"),
             ],
         ),
     )
@@ -103,23 +160,33 @@ def create_simple_model_tables(mf_test_session_state: MetricFlowTestSessionState
                 ("l3141592", "us", 3, True, "u0004114", "2020-01-01"),
                 ("l5948301", "us", 5, True, "u0004114", "2020-01-02"),
                 ("l2718281", cote_divoire, 4, False, "u0005432", "2020-01-02"),
+                ("l9658588-incomplete", "us", None, None, "u1004114", "2020-01-02"),
+                ("l8912456-incomplete", None, None, None, "u1004114", "2020-01-02"),
+                ("l7891283-incomplete", "ca", None, False, "u1004114", "2020-01-02"),
             ],
         ),
     )
 
+    # The dim_listings table is an SCD type II table in the style of a dbt snapshot with two columns setting the
+    # the range of time when this row was "active", with a NULL value for "active_to" meaning it's still current
     create_table(
         sql_client=sql_client,
         sql_table=SqlTable(schema_name=schema, table_name="dim_listings"),
         df=make_df(
             sql_client=sql_client,
-            columns=["listing_id", "country", "capacity", "is_lux", DEFAULT_DS],
-            time_columns={DEFAULT_DS},
+            columns=["listing_id", "country", "capacity", "is_lux", "user_id", "active_from", "active_to"],
+            time_columns={"active_from", "active_to"},
             data=[
-                ("l3141592", "us", 2, True, "2020-01-01"),
-                ("l2718281", cote_divoire, 4, True, "2020-01-01"),
-                ("l3141592", "us", 3, True, "2020-01-02"),
-                ("l2718281", cote_divoire, 4, False, "2020-01-02"),
-                ("l5948301", "us", 5, True, "2020-01-02"),
+                ("l3141592", "us", 2, True, "u0004114", "2020-01-01", "2020-01-02"),
+                ("l2718281", cote_divoire, 4, True, "u0005432", "2020-01-01", "2020-01-02"),
+                ("l3141592", "us", 3, True, "u0004114", "2020-01-02", None),
+                # This cote_divoire property changed hands to a person from Maryland who considers it not lux
+                ("l2718281", cote_divoire, 4, False, "u0003154", "2020-01-02", None),
+                ("l5948301", "us", 5, True, "u0004114", "2020-01-02", None),
+                ("l9658588-incomplete", None, None, None, "u1004114", "2020-01-01", "2020-01-02"),
+                ("l9658588-incomplete", "us", None, None, "u1004114", "2020-01-02", None),
+                ("l8912456-incomplete", None, None, None, "u1004114", "2020-01-02", None),
+                ("l7891283-incomplete", "ca", None, False, "u1004114", "2020-01-02", None),
             ],
         ),
     )
@@ -210,6 +277,23 @@ def create_simple_model_tables(mf_test_session_state: MetricFlowTestSessionState
             data=[
                 ("l3141592", "ll_001"),
                 ("l5948301", "LUX_TEST_ID"),
+                ("l2718281", "ll_002"),
+            ],
+        ),
+    )
+
+    create_table(
+        sql_client=sql_client,
+        sql_table=SqlTable(schema_name=schema, table_name="dim_lux_listings"),
+        df=make_df(
+            sql_client=sql_client,
+            columns=["lux_listing_id", "is_confirmed_lux", "valid_from", "valid_to"],
+            time_columns={"valid_from", "valid_to"},
+            data=[
+                ("ll_001", True, "2020-01-01", None),
+                ("LUX_TEST_ID", True, "2020-01-02", None),
+                ("ll_002", True, "2020-01-01", "2020-01-02"),
+                ("ll_002", False, "2020-01-02", None),
             ],
         ),
     )
@@ -226,9 +310,69 @@ def create_simple_model_tables(mf_test_session_state: MetricFlowTestSessionState
         ),
     )
 
+    accounts_data = [
+        ("u0004114", 2135, "2020-01-01", "checking"),
+        ("u0004114", 1234, "2020-01-01", "savings"),
+        ("u0005432", 5234, "2020-01-03", "checking"),
+        ("u0005432", 234, "2020-01-03", "savings"),
+        ("u0003141", 24634, "2020-01-01", "checking"),
+        ("u0003154", 23452, "2020-01-01", "checking"),
+        ("u0004114", 1213, "2020-01-06", "savings"),
+        ("u1612112", 5123, "2020-01-02", "checking"),
+        ("u0004114", 523, "2020-01-07", "checking"),
+        ("u0004114", 7434, "2020-01-10", "checkings"),
+        ("u0005432", 8456, "2020-01-02", "savings"),
+        ("u0003141", 12939, "2020-01-12", "checking"),
+        ("u0003452", 6939, "2020-01-02", "checking"),
+        ("u0003452", 35, "2020-01-12", "checking"),
+        ("u0005414", 5582, "2020-01-04", "savings"),
+    ]
+
+    create_table(
+        sql_client=sql_client,
+        sql_table=SqlTable(schema_name=schema, table_name="fct_accounts"),
+        df=make_df(
+            sql_client=sql_client,
+            columns=["user_id", "account_balance", DEFAULT_DS, "account_type"],
+            time_columns={DEFAULT_DS},
+            data=accounts_data,
+        ),
+    )
+
+    create_table(
+        sql_client=sql_client,
+        sql_table=SqlTable(schema_name=schema, table_name="dim_primary_accounts"),
+        df=make_df(
+            sql_client=sql_client,
+            columns=["user_id", "account_type", "set_as_primary", "removed_as_primary"],
+            time_columns={"set_as_primary", "removed_as_primary"},
+            data=[
+                ("u0004114", "savings", "2020-01-01", "2020-01-02"),
+                ("u1612112", "checking", "2020-01-01", "2020-01-01"),
+                ("u0005432", "savings", "2020-01-02", "2020-01-03"),
+                ("u0003452", "checking", "2020-01-01", None),
+                ("u0003154", "checking", "2020-01-01", None),
+                ("u0003141", "checking", "2020-01-01", None),
+                ("u0004114", "savings", "2020-01-02", None),
+                ("u1612112", "checking", "2020-01-02", "2020-01-02"),
+                ("u0005432", "checking", "2020-01-03", None),
+            ],
+        ),
+    )
+
     return True
 
 
+@pytest.fixture(scope="session")
+def create_scd_model_tables(create_simple_model_tables: bool) -> bool:
+    """This fixture ensures the scd tables are created when necessary
+
+    The SCD model relies on the simple model tables, so we simply depend on that fixture and return it here.
+    If, at any point, we decide to diverge from the simple model we can update this fixture independently.
+    """
+    return create_simple_model_tables
+
+
 @pytest.fixture(scope="session")
 def create_bridge_table(mf_test_session_state: MetricFlowTestSessionState, sql_client: SqlClient) -> bool:
     """Creates tables with example source data for multi-hop join testing."""
@@ -488,3 +632,27 @@ def create_extended_date_model_tables(mf_test_session_state: MetricFlowTestSessi
     )
 
     return True
+
+
+@pytest.fixture(scope="session")
+def create_data_warehouse_validation_model_tables(
+    mf_test_session_state: MetricFlowTestSessionState, sql_client: SqlClient
+) -> bool:
+    """Creates tables with example source data for testing."""
+    schema = mf_test_session_state.mf_source_schema
+
+    data = [
+        (True, "2022-06-01"),
+    ]
+    create_table(
+        sql_client=sql_client,
+        sql_table=SqlTable(schema_name=schema, table_name="fct_animals"),
+        df=make_df(
+            sql_client=sql_client,
+            columns=["is_dog", DEFAULT_DS],
+            time_columns={DEFAULT_DS},
+            data=data,
+        ),
+    )
+
+    return True
