@@ -6,10 +6,11 @@ from metricflow.plan_conversion.instance_converters import (
     FilterElements,
 )
 from metricflow.plan_conversion.select_column_gen import SelectColumnSet
-from metricflow.specs import MeasureSpec, MetricInputMeasureSpec
+from metricflow.specs import MeasureSpec, MetricInputMeasureSpec, InstanceSpecSet
 from metricflow.sql.sql_exprs import (
     SqlFunction,
-    SqlFunctionExpression,
+    SqlAggregateFunctionExpression,
+    SqlPercentileExpression,
 )
 from metricflow.test.fixtures.model_fixtures import ConsistentIdObjectRepository
 
@@ -22,10 +23,10 @@ def __get_filtered_measure_instance_set(
     """Gets an InstanceSet consisting of only the measure instance matching the given name and data source"""
     dataset = object_repo.simple_model_data_sets[data_source_name]
     instance_set = dataset.instance_set
-    include_specs = [
+    include_specs = tuple(
         instance.spec for instance in instance_set.measure_instances if instance.spec.element_name == measure_name
-    ]
-    return FilterElements(include_specs=include_specs).transform(instance_set)
+    )
+    return FilterElements(include_specs=InstanceSpecSet(measure_specs=include_specs)).transform(instance_set)
 
 
 def test_sum_aggregation(
@@ -46,7 +47,7 @@ def test_sum_aggregation(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     assert expr.sql_function == SqlFunction.SUM
 
 
@@ -68,7 +69,7 @@ def test_sum_boolean_aggregation(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     # The SUM_BOOLEAN aggregation type is transformed to SUM at model parsing time
     assert expr.sql_function == SqlFunction.SUM
 
@@ -91,7 +92,7 @@ def test_avg_aggregation(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     assert expr.sql_function == SqlFunction.AVERAGE
 
 
@@ -113,7 +114,7 @@ def test_count_distinct_aggregation(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     assert expr.sql_function == SqlFunction.COUNT_DISTINCT
 
 
@@ -135,7 +136,7 @@ def test_max_aggregation(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     assert expr.sql_function == SqlFunction.MAX
 
 
@@ -157,7 +158,7 @@ def test_min_aggregation(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     assert expr.sql_function == SqlFunction.MIN
 
 
@@ -179,6 +180,28 @@ def test_aliased_sum(
     assert len(select_column_set.measure_columns) == 1
     measure_column = select_column_set.measure_columns[0]
     expr = measure_column.expr
-    assert isinstance(expr, SqlFunctionExpression)
+    assert isinstance(expr, SqlAggregateFunctionExpression)
     assert expr.sql_function == SqlFunction.SUM
     assert measure_column.column_alias == "bvalue"
+
+
+def test_percentile_aggregation(
+    consistent_id_object_repository: ConsistentIdObjectRepository,
+    simple_semantic_model: SemanticModel,
+) -> None:
+    """Checks for function expression handling for booking_value, a percentile type metric in the simple model"""
+    measure_name = "booking_value_p99"
+    instance_set = __get_filtered_measure_instance_set("bookings_source", measure_name, consistent_id_object_repository)
+
+    select_column_set: SelectColumnSet = CreateSelectColumnsWithMeasuresAggregated(
+        __SOURCE_TABLE_ALIAS,
+        DefaultColumnAssociationResolver(simple_semantic_model),
+        simple_semantic_model.data_source_semantics,
+        (MetricInputMeasureSpec(measure_spec=MeasureSpec(element_name="booking_value_p99")),),
+    ).transform(instance_set=instance_set)
+
+    assert len(select_column_set.measure_columns) == 1
+    measure_column = select_column_set.measure_columns[0]
+    expr = measure_column.expr
+    assert isinstance(expr, SqlPercentileExpression)
+    assert expr.percentile_args.percentile == 0.99
