@@ -7,7 +7,7 @@ from metricflow.plan_conversion.column_resolver import DefaultColumnAssociationR
 from metricflow.plan_conversion.dataflow_to_execution import DataflowToExecutionPlanConverter
 from metricflow.plan_conversion.dataflow_to_sql import DataflowToSqlQueryPlanConverter
 from metricflow.plan_conversion.time_spine import TimeSpineSource
-from metricflow.protocols.sql_client import SqlClient
+from metricflow.protocols.async_sql_client import AsyncSqlClient
 from metricflow.specs import (
     MetricFlowQuerySpec,
     MetricSpec,
@@ -22,7 +22,7 @@ from metricflow.test.plan_utils import assert_execution_plan_text_equal
 
 def make_execution_plan_converter(  # noqa: D
     semantic_model: SemanticModel,
-    sql_client: SqlClient,
+    sql_client: AsyncSqlClient,
     time_spine_source: TimeSpineSource,
 ) -> DataflowToExecutionPlanConverter:
     return DataflowToExecutionPlanConverter[DataSourceDataSet](
@@ -41,7 +41,7 @@ def test_joined_plan(  # noqa: D
     mf_test_session_state: MetricFlowTestSessionState,
     dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
     simple_semantic_model: SemanticModel,
-    sql_client: SqlClient,
+    async_sql_client: AsyncSqlClient,
     time_spine_source: TimeSpineSource,
 ) -> None:
     dataflow_plan = dataflow_plan_builder.build_plan(
@@ -60,14 +60,16 @@ def test_joined_plan(  # noqa: D
         )
     )
 
-    to_execution_plan_converter = make_execution_plan_converter(simple_semantic_model, sql_client, time_spine_source)
+    to_execution_plan_converter = make_execution_plan_converter(
+        simple_semantic_model, async_sql_client, time_spine_source
+    )
 
     execution_plan = to_execution_plan_converter.convert_to_execution_plan(dataflow_plan)
 
     assert_execution_plan_text_equal(
         request=request,
         mf_test_session_state=mf_test_session_state,
-        sql_client=sql_client,
+        sql_client=async_sql_client,
         execution_plan=execution_plan,
     )
 
@@ -75,7 +77,7 @@ def test_joined_plan(  # noqa: D
 def test_small_combined_metrics_plan(  # noqa: D
     request: FixtureRequest,
     mf_test_session_state: MetricFlowTestSessionState,
-    sql_client: SqlClient,
+    async_sql_client: AsyncSqlClient,
     dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
     simple_semantic_model: SemanticModel,
     time_spine_source: TimeSpineSource,
@@ -97,7 +99,7 @@ def test_small_combined_metrics_plan(  # noqa: D
 
     to_execution_plan_converter = make_execution_plan_converter(
         semantic_model=simple_semantic_model,
-        sql_client=sql_client,
+        sql_client=async_sql_client,
         time_spine_source=time_spine_source,
     )
     execution_plan = to_execution_plan_converter.convert_to_execution_plan(dataflow_plan)
@@ -105,7 +107,7 @@ def test_small_combined_metrics_plan(  # noqa: D
     assert_execution_plan_text_equal(
         request=request,
         mf_test_session_state=mf_test_session_state,
-        sql_client=sql_client,
+        sql_client=async_sql_client,
         execution_plan=execution_plan,
     )
 
@@ -113,7 +115,7 @@ def test_small_combined_metrics_plan(  # noqa: D
 def test_combined_metrics_plan(  # noqa: D
     request: FixtureRequest,
     mf_test_session_state: MetricFlowTestSessionState,
-    sql_client: SqlClient,
+    async_sql_client: AsyncSqlClient,
     dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
     simple_semantic_model: SemanticModel,
     time_spine_source: TimeSpineSource,
@@ -137,7 +139,7 @@ def test_combined_metrics_plan(  # noqa: D
 
     to_execution_plan_converter = make_execution_plan_converter(
         semantic_model=simple_semantic_model,
-        sql_client=sql_client,
+        sql_client=async_sql_client,
         time_spine_source=time_spine_source,
     )
     execution_plan = to_execution_plan_converter.convert_to_execution_plan(dataflow_plan)
@@ -145,7 +147,7 @@ def test_combined_metrics_plan(  # noqa: D
     assert_execution_plan_text_equal(
         request=request,
         mf_test_session_state=mf_test_session_state,
-        sql_client=sql_client,
+        sql_client=async_sql_client,
         execution_plan=execution_plan,
     )
 
@@ -155,7 +157,7 @@ def test_multihop_joined_plan(  # noqa: D
     mf_test_session_state: MetricFlowTestSessionState,
     multihop_dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
     multi_hop_join_semantic_model: SemanticModel,
-    sql_client: SqlClient,
+    async_sql_client: AsyncSqlClient,
     time_spine_source: TimeSpineSource,
 ) -> None:
     """Tests a plan getting a measure and a joined dimension."""
@@ -181,7 +183,7 @@ def test_multihop_joined_plan(  # noqa: D
             time_spine_source=time_spine_source,
         ),
         sql_plan_renderer=DefaultSqlQueryPlanRenderer(),
-        sql_client=sql_client,
+        sql_client=async_sql_client,
     )
 
     execution_plan = to_execution_plan_converter.convert_to_execution_plan(dataflow_plan)
@@ -189,6 +191,6 @@ def test_multihop_joined_plan(  # noqa: D
     assert_execution_plan_text_equal(
         request=request,
         mf_test_session_state=mf_test_session_state,
-        sql_client=sql_client,
+        sql_client=async_sql_client,
         execution_plan=execution_plan,
     )
