@@ -5,7 +5,7 @@ SELECT
   , CAST(subq_19.bookings AS FLOAT64) / CAST(NULLIF(subq_19.views, 0) AS FLOAT64) AS bookings_per_view
 FROM (
   -- Pass Only Elements:
-  --   ['listing__country_latest', 'ds', 'bookings', 'views']
+  --   ['bookings', 'views', 'listing__country_latest', 'ds']
   SELECT
     subq_18.ds
     , subq_18.listing__country_latest
@@ -100,6 +100,11 @@ FROM (
                 , subq_0.bookers
                 , subq_0.average_booking_value
                 , subq_0.referred_bookings
+                , subq_0.median_booking_value
+                , subq_0.booking_value_p99
+                , subq_0.discrete_booking_value_p99
+                , subq_0.approximate_continuous_booking_value_p99
+                , subq_0.approximate_discrete_booking_value_p99
               FROM (
                 -- Read Elements From Data Source 'bookings_source'
                 SELECT
@@ -112,6 +117,11 @@ FROM (
                   , bookings_source_src_10001.booking_value AS average_booking_value
                   , bookings_source_src_10001.booking_value AS booking_payments
                   , CASE WHEN referrer_id IS NOT NULL THEN 1 ELSE 0 END AS referred_bookings
+                  , bookings_source_src_10001.booking_value AS median_booking_value
+                  , bookings_source_src_10001.booking_value AS booking_value_p99
+                  , bookings_source_src_10001.booking_value AS discrete_booking_value_p99
+                  , bookings_source_src_10001.booking_value AS approximate_continuous_booking_value_p99
+                  , bookings_source_src_10001.booking_value AS approximate_discrete_booking_value_p99
                   , bookings_source_src_10001.is_instant
                   , bookings_source_src_10001.ds
                   , DATE_TRUNC(bookings_source_src_10001.ds, isoweek) AS ds__week
@@ -160,7 +170,7 @@ FROM (
           ) subq_2
           LEFT OUTER JOIN (
             -- Pass Only Elements:
-            --   ['listing', 'country_latest']
+            --   ['country_latest', 'listing']
             SELECT
               subq_4.listing
               , subq_4.country_latest
@@ -248,8 +258,8 @@ FROM (
         ) subq_6
       ) subq_7
       GROUP BY
-        subq_7.ds
-        , subq_7.listing__country_latest
+        ds
+        , listing__country_latest
     ) subq_8
     INNER JOIN (
       -- Aggregate Measures
@@ -350,7 +360,7 @@ FROM (
           ) subq_11
           LEFT OUTER JOIN (
             -- Pass Only Elements:
-            --   ['listing', 'country_latest']
+            --   ['country_latest', 'listing']
             SELECT
               subq_13.listing
               , subq_13.country_latest
@@ -438,11 +448,17 @@ FROM (
         ) subq_15
       ) subq_16
       GROUP BY
-        subq_16.ds
-        , subq_16.listing__country_latest
+        ds
+        , listing__country_latest
     ) subq_17
     ON
       (
+        (
+          subq_8.ds = subq_17.ds
+        ) OR (
+          (subq_8.ds IS NULL) AND (subq_17.ds IS NULL)
+        )
+      ) AND (
         (
           subq_8.listing__country_latest = subq_17.listing__country_latest
         ) OR (
@@ -452,12 +468,6 @@ FROM (
             subq_17.listing__country_latest IS NULL
           )
         )
-      ) AND (
-        (
-          subq_8.ds = subq_17.ds
-        ) OR (
-          (subq_8.ds IS NULL) AND (subq_17.ds IS NULL)
-        )
       )
   ) subq_18
 ) subq_19
