@@ -1,34 +1,34 @@
 -- Compute Metrics via Expressions
 SELECT
-  CAST(bookings AS FLOAT64) / CAST(NULLIF(bookers, 0) AS FLOAT64) AS bookings_per_booker
+  listing
   , listing__country_latest
-  , listing
+  , CAST(bookings AS FLOAT64) / CAST(NULLIF(bookers, 0) AS FLOAT64) AS bookings_per_booker
 FROM (
   -- Join Standard Outputs
   -- Aggregate Measures
   SELECT
-    SUM(subq_7.bookings) AS bookings
+    subq_7.listing AS listing
+    , listings_latest_src_10004.country AS listing__country_latest
+    , SUM(subq_7.bookings) AS bookings
     , COUNT(DISTINCT subq_7.bookers) AS bookers
-    , listings_latest_src_10003.country AS listing__country_latest
-    , subq_7.listing AS listing
   FROM (
     -- Read Elements From Data Source 'bookings_source'
     -- Pass Only Elements:
     --   ['bookings', 'bookers', 'listing']
     SELECT
-      1 AS bookings
+      listing_id AS listing
+      , 1 AS bookings
       , guest_id AS bookers
-      , listing_id AS listing
     FROM (
       -- User Defined SQL Query
       SELECT * FROM ***************************.fct_bookings
-    ) bookings_source_src_10000
+    ) bookings_source_src_10001
   ) subq_7
   LEFT OUTER JOIN
-    ***************************.dim_listings_latest listings_latest_src_10003
+    ***************************.dim_listings_latest listings_latest_src_10004
   ON
-    subq_7.listing = listings_latest_src_10003.listing_id
+    subq_7.listing = listings_latest_src_10004.listing_id
   GROUP BY
-    listing__country_latest
-    , listing
+    listing
+    , listing__country_latest
 ) subq_11
