@@ -4,7 +4,7 @@ SELECT
   , CAST(subq_11.booking_value_with_is_instant_constraint AS FLOAT64) / CAST(NULLIF(subq_11.booking_value, 0) AS FLOAT64) AS instant_booking_value_ratio
 FROM (
   -- Pass Only Elements:
-  --   ['metric_time', 'booking_value_with_is_instant_constraint', 'booking_value']
+  --   ['booking_value_with_is_instant_constraint', 'booking_value', 'metric_time']
   SELECT
     subq_10.metric_time
     , subq_10.booking_value_with_is_instant_constraint
@@ -94,6 +94,11 @@ FROM (
                 , subq_0.bookers
                 , subq_0.average_booking_value
                 , subq_0.referred_bookings
+                , subq_0.median_booking_value
+                , subq_0.booking_value_p99
+                , subq_0.discrete_booking_value_p99
+                , subq_0.approximate_continuous_booking_value_p99
+                , subq_0.approximate_discrete_booking_value_p99
               FROM (
                 -- Read Elements From Data Source 'bookings_source'
                 SELECT
@@ -106,6 +111,11 @@ FROM (
                   , bookings_source_src_10001.booking_value AS average_booking_value
                   , bookings_source_src_10001.booking_value AS booking_payments
                   , CASE WHEN referrer_id IS NOT NULL THEN 1 ELSE 0 END AS referred_bookings
+                  , bookings_source_src_10001.booking_value AS median_booking_value
+                  , bookings_source_src_10001.booking_value AS booking_value_p99
+                  , bookings_source_src_10001.booking_value AS discrete_booking_value_p99
+                  , bookings_source_src_10001.booking_value AS approximate_continuous_booking_value_p99
+                  , bookings_source_src_10001.booking_value AS approximate_discrete_booking_value_p99
                   , bookings_source_src_10001.is_instant
                   , bookings_source_src_10001.ds
                   , DATE_TRUNC(bookings_source_src_10001.ds, isoweek) AS ds__week
@@ -156,7 +166,7 @@ FROM (
         ) subq_3
       ) subq_4
       GROUP BY
-        subq_4.metric_time
+        metric_time
     ) subq_5
     INNER JOIN (
       -- Aggregate Measures
@@ -224,6 +234,11 @@ FROM (
             , subq_6.bookers
             , subq_6.average_booking_value
             , subq_6.referred_bookings
+            , subq_6.median_booking_value
+            , subq_6.booking_value_p99
+            , subq_6.discrete_booking_value_p99
+            , subq_6.approximate_continuous_booking_value_p99
+            , subq_6.approximate_discrete_booking_value_p99
           FROM (
             -- Read Elements From Data Source 'bookings_source'
             SELECT
@@ -236,6 +251,11 @@ FROM (
               , bookings_source_src_10001.booking_value AS average_booking_value
               , bookings_source_src_10001.booking_value AS booking_payments
               , CASE WHEN referrer_id IS NOT NULL THEN 1 ELSE 0 END AS referred_bookings
+              , bookings_source_src_10001.booking_value AS median_booking_value
+              , bookings_source_src_10001.booking_value AS booking_value_p99
+              , bookings_source_src_10001.booking_value AS discrete_booking_value_p99
+              , bookings_source_src_10001.booking_value AS approximate_continuous_booking_value_p99
+              , bookings_source_src_10001.booking_value AS approximate_discrete_booking_value_p99
               , bookings_source_src_10001.is_instant
               , bookings_source_src_10001.ds
               , DATE_TRUNC(bookings_source_src_10001.ds, isoweek) AS ds__week
@@ -283,15 +303,13 @@ FROM (
         ) subq_7
       ) subq_8
       GROUP BY
-        subq_8.metric_time
+        metric_time
     ) subq_9
     ON
       (
-        (
-          subq_5.metric_time = subq_9.metric_time
-        ) OR (
-          (subq_5.metric_time IS NULL) AND (subq_9.metric_time IS NULL)
-        )
+        subq_5.metric_time = subq_9.metric_time
+      ) OR (
+        (subq_5.metric_time IS NULL) AND (subq_9.metric_time IS NULL)
       )
   ) subq_10
 ) subq_11
