@@ -95,6 +95,11 @@ FROM (
                 , subq_0.bookers
                 , subq_0.average_booking_value
                 , subq_0.referred_bookings
+                , subq_0.median_booking_value
+                , subq_0.booking_value_p99
+                , subq_0.discrete_booking_value_p99
+                , subq_0.approximate_continuous_booking_value_p99
+                , subq_0.approximate_discrete_booking_value_p99
               FROM (
                 -- Read Elements From Data Source 'bookings_source'
                 SELECT
@@ -107,6 +112,11 @@ FROM (
                   , bookings_source_src_10001.booking_value AS average_booking_value
                   , bookings_source_src_10001.booking_value AS booking_payments
                   , CASE WHEN referrer_id IS NOT NULL THEN 1 ELSE 0 END AS referred_bookings
+                  , bookings_source_src_10001.booking_value AS median_booking_value
+                  , bookings_source_src_10001.booking_value AS booking_value_p99
+                  , bookings_source_src_10001.booking_value AS discrete_booking_value_p99
+                  , bookings_source_src_10001.booking_value AS approximate_continuous_booking_value_p99
+                  , bookings_source_src_10001.booking_value AS approximate_discrete_booking_value_p99
                   , bookings_source_src_10001.is_instant
                   , bookings_source_src_10001.ds
                   , DATE_TRUNC('week', bookings_source_src_10001.ds) AS ds__week
@@ -155,7 +165,7 @@ FROM (
           ) subq_2
           LEFT OUTER JOIN (
             -- Pass Only Elements:
-            --   ['listing', 'country_latest']
+            --   ['country_latest', 'listing']
             SELECT
               subq_4.listing
               , subq_4.country_latest
