@@ -11,7 +11,7 @@ from metricflow.dataset.dataset import DataSet
 from metricflow.model.semantic_model import SemanticModel
 from metricflow.plan_conversion.column_resolver import DefaultColumnAssociationResolver
 from metricflow.plan_conversion.time_spine import TimeSpineSource
-from metricflow.specs import MetricSpec
+from metricflow.references import MetricReference
 from metricflow.test.time.metric_time_dimension import MTD_SPEC_DAY, MTD_SPEC_MONTH, MTD_SPEC_YEAR, MTD_REFERENCE
 from metricflow.time.time_granularity import TimeGranularity
 from metricflow.time.time_granularity_solver import (
@@ -49,21 +49,21 @@ def time_granularity_solver(  # noqa: D
 
 def test_validate_day_granuarity_for_day_metric(time_granularity_solver: TimeGranularitySolver) -> None:  # noqa: D
     time_granularity_solver.validate_time_granularity(
-        metric_specs=[MetricSpec(element_name="bookings")],
+        metric_references=[MetricReference(element_name="bookings")],
         time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.DAY)],
     )
 
 
 def test_validate_month_granuarity_for_day_metric(time_granularity_solver: TimeGranularitySolver) -> None:  # noqa: D
     time_granularity_solver.validate_time_granularity(
-        metric_specs=[MetricSpec(element_name="bookings")],
+        metric_references=[MetricReference(element_name="bookings")],
         time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.MONTH)],
     )
 
 
 def test_validate_month_granuarity_for_month_metric(time_granularity_solver: TimeGranularitySolver) -> None:  # noqa: D
     time_granularity_solver.validate_time_granularity(
-        metric_specs=[MetricSpec(element_name="bookings_monthly")],
+        metric_references=[MetricReference(element_name="bookings_monthly")],
         time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.MONTH)],
     )
 
@@ -72,7 +72,7 @@ def test_validate_month_granuarity_for_day_and_month_metrics(  # noqa: D
     time_granularity_solver: TimeGranularitySolver,
 ) -> None:
     time_granularity_solver.validate_time_granularity(
-        metric_specs=[MetricSpec(element_name="bookings"), MetricSpec(element_name="bookings_monthly")],
+        metric_references=[MetricReference(element_name="bookings"), MetricReference(element_name="bookings_monthly")],
         time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.MONTH)],
     )
 
@@ -81,7 +81,7 @@ def test_validate_year_granularity_for_day_and_month_metrics(  # noqa: D
     time_granularity_solver: TimeGranularitySolver,
 ) -> None:
     time_granularity_solver.validate_time_granularity(
-        metric_specs=[MetricSpec(element_name="bookings"), MetricSpec(element_name="bookings_monthly")],
+        metric_references=[MetricReference(element_name="bookings"), MetricReference(element_name="bookings_monthly")],
         time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.YEAR)],
     )
 
@@ -89,7 +89,7 @@ def test_validate_year_granularity_for_day_and_month_metrics(  # noqa: D
 def test_validate_day_granuarity_for_month_metric(time_granularity_solver: TimeGranularitySolver) -> None:  # noqa: D
     with pytest.raises(RequestTimeGranularityException):
         time_granularity_solver.validate_time_granularity(
-            metric_specs=[MetricSpec(element_name="bookings_monthly")],
+            metric_references=[MetricReference(element_name="bookings_monthly")],
             time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.DAY)],
         )
 
@@ -99,7 +99,10 @@ def test_validate_day_granularity_for_day_and_month_metric(  # noqa: D
 ) -> None:
     with pytest.raises(RequestTimeGranularityException):
         time_granularity_solver.validate_time_granularity(
-            metric_specs=[MetricSpec(element_name="bookings"), MetricSpec(element_name="bookings_monthly")],
+            metric_references=[
+                MetricReference(element_name="bookings"),
+                MetricReference(element_name="bookings_monthly"),
+            ],
             time_dimension_specs=[DataSet.metric_time_dimension_spec(TimeGranularity.DAY)],
         )
 
@@ -109,7 +112,7 @@ PARTIAL_PTD_SPEC = PartialTimeDimensionSpec(element_name=DataSet.metric_time_dim
 
 def test_granularity_solution_for_day_metric(time_granularity_solver: TimeGranularitySolver) -> None:  # noqa: D
     assert time_granularity_solver.resolve_granularity_for_partial_time_dimension_specs(
-        metric_specs=[MetricSpec(element_name="bookings")],
+        metric_references=[MetricReference(element_name="bookings")],
         partial_time_dimension_specs=[PARTIAL_PTD_SPEC],
         metric_time_dimension_reference=MTD_REFERENCE,
     ) == {
@@ -119,7 +122,7 @@ def test_granularity_solution_for_day_metric(time_granularity_solver: TimeGranul
 
 def test_granularity_solution_for_month_metric(time_granularity_solver: TimeGranularitySolver) -> None:  # noqa: D
     assert time_granularity_solver.resolve_granularity_for_partial_time_dimension_specs(
-        metric_specs=[MetricSpec(element_name="bookings_monthly")],
+        metric_references=[MetricReference(element_name="bookings_monthly")],
         partial_time_dimension_specs=[PARTIAL_PTD_SPEC],
         metric_time_dimension_reference=MTD_REFERENCE,
     ) == {
@@ -131,17 +134,30 @@ def test_granularity_solution_for_day_and_month_metrics(  # noqa: D
     time_granularity_solver: TimeGranularitySolver,
 ) -> None:
     assert time_granularity_solver.resolve_granularity_for_partial_time_dimension_specs(
-        metric_specs=[MetricSpec(element_name="bookings"), MetricSpec(element_name="bookings_monthly")],
+        metric_references=[MetricReference(element_name="bookings"), MetricReference(element_name="bookings_monthly")],
         partial_time_dimension_specs=[PARTIAL_PTD_SPEC],
         metric_time_dimension_reference=MTD_REFERENCE,
     ) == {PARTIAL_PTD_SPEC: MTD_SPEC_MONTH}
 
 
+def test_granularity_error_for_cumulative_metric(  # noqa: D
+    time_granularity_solver: TimeGranularitySolver,
+) -> None:
+    with pytest.raises(RequestTimeGranularityException):
+        time_granularity_solver.validate_time_granularity(
+            metric_references=[
+                MetricReference(element_name="weekly_bookers"),
+                MetricReference(element_name="bookings_monthly"),
+            ],
+            time_dimension_specs=[MTD_SPEC_MONTH],
+        )
+
+
 def test_time_granularity_parameter(  # noqa: D
     time_granularity_solver: TimeGranularitySolver,
 ) -> None:
     assert time_granularity_solver.resolve_granularity_for_partial_time_dimension_specs(
-        metric_specs=[MetricSpec(element_name="bookings"), MetricSpec(element_name="bookings_monthly")],
+        metric_references=[MetricReference(element_name="bookings"), MetricReference(element_name="bookings_monthly")],
         partial_time_dimension_specs=[PARTIAL_PTD_SPEC],
         metric_time_dimension_reference=MTD_REFERENCE,
         time_granularity=TimeGranularity.YEAR,
@@ -153,7 +169,10 @@ def test_invalid_time_granularity_parameter(  # noqa: D
 ) -> None:
     with pytest.raises(RequestTimeGranularityException, match="Can't use time granularity.*"):
         time_granularity_solver.resolve_granularity_for_partial_time_dimension_specs(
-            metric_specs=[MetricSpec(element_name="bookings"), MetricSpec(element_name="bookings_monthly")],
+            metric_references=[
+                MetricReference(element_name="bookings"),
+                MetricReference(element_name="bookings_monthly"),
+            ],
             partial_time_dimension_specs=[PARTIAL_PTD_SPEC],
             metric_time_dimension_reference=MTD_REFERENCE,
             time_granularity=TimeGranularity.DAY,
@@ -218,10 +237,18 @@ def test_adjusted_time_constraint_at_boundaries(
     assert (
         time_granularity_solver.adjust_time_range_to_granularity(
             time_range_constraint=TimeRangeConstraint(
-                start_time=TimeRangeConstraint.all_time().start_time + datetime.timedelta(days=-30),
-                end_time=TimeRangeConstraint.all_time().end_time + datetime.timedelta(days=30),
+                start_time=TimeRangeConstraint.all_time().start_time + datetime.timedelta(days=1),
+                end_time=TimeRangeConstraint.all_time().end_time,
             ),
             time_granularity=TimeGranularity.WEEK,
         )
         == TimeRangeConstraint.all_time()
     )
+
+    assert (
+        time_granularity_solver.adjust_time_range_to_granularity(
+            time_range_constraint=TimeRangeConstraint.all_time(),
+            time_granularity=TimeGranularity.WEEK,
+        )
+        == TimeRangeConstraint.all_time()
+    )
